"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopService = exports.transform = void 0;
const esbuild_1 = require("esbuild");
const mdx_1 = __importDefault(require("@mdx-js/mdx"));
const find_dependency_1 = __importDefault(require("find-dependency"));
const pluginName = 'vite-plugin-mdx';
function transform(code_mdx, mdxOptions, root = __dirname) {
    return __awaiter(this, void 0, void 0, function* () {
        const code_jsx = yield mdx_1.default(code_mdx, mdxOptions);
        const code_es2019 = yield jsxToES2019(code_jsx);
        const code_final = injectImports(code_es2019, root);
        return code_final;
    });
}
exports.transform = transform;
function jsxToES2019(code_jsx) {
    return __awaiter(this, void 0, void 0, function* () {
        // We use `esbuild` ourselves instead of letting Vite doing the esbuild transform,
        // because there don't seem to be a way to change the esbuild options for specific
        // files only: https://vitejs.dev/config/#esbuild
        const esBuild = yield ensureEsbuildService();
        /* Uncomment this to inspect the type `TransformOptions`
        type TransformOptions = Pick<Parameters<typeof esBuild.transform>, 1>[1];
        let t: TransformOptions;
        t!.format
        t!.jsxFactory
        //*/
        let { code: code_es2019 } = yield esBuild.transform(code_jsx, {
            loader: 'jsx',
            jsxFactory: 'mdx',
            target: 'es2019'
        });
        // TODO stabilize this bugfix
        code_es2019 = code_es2019.replace('export default function MDXContent', 'export default MDXContent; function MDXContent');
        return code_es2019;
    });
}
function injectImports(code_es2019, root) {
    if (findPackage('preact', root)) {
        return [
            `import { h } from 'preact'`,
            `import { mdx } from '${getMdxImportPath('@mdx-js/preact', root)}'`,
            '',
            code_es2019
        ].join('\n');
    }
    if (findPackage('react', root)) {
        return [
            `import React from 'react'`,
            `import { mdx } from '${getMdxImportPath('@mdx-js/react', root)}'`,
            '',
            code_es2019
        ].join('\n');
    }
    throw new Error(`[Wrong Usage][${pluginName}] You need to \`npm install react\` or \`npm install preact\`.`);
}
function getMdxImportPath(mdxPackageName, root) {
    const mdxPackageRoot = findPackage(mdxPackageName, root);
    if (mdxPackageRoot) {
        return mdxPackageName;
    }
    throw new Error(`[Wrong Usage][${pluginName}] You need to \`npm install ${mdxPackageName}\`.`);
}
/**
 * Search the node_modules of `cwd` and its ancestors until a package is found.
 * Skip global node_modules and vite/node_modules (local clone might be used).
 */
function findPackage(name, cwd) {
    return find_dependency_1.default(name, { cwd, skipGlobal: true });
}
let _service;
function ensureEsbuildService() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!_service) {
            _service = esbuild_1.startService();
        }
        return _service;
    });
}
function stopService() {
    return __awaiter(this, void 0, void 0, function* () {
        if (_service) {
            const service = yield _service;
            service.stop();
            _service = undefined;
        }
    });
}
exports.stopService = stopService;
//# sourceMappingURL=transform.js.map